#!/bin/bash

##################################################
# Valid connectivity check for MQTT sessions
# Checks TCP connection is established given some context and considerations
# May take recovery actions
# Returns 0 if no further action is needed, 1 if reboot is advised
#
# (c) 2019 hector.cordobes@imdea.org
#
##################################################

# Number of failures detected before taking action
THRESHOLD=5
# Remote service check timeout (in seconds)
TIMEOUT=2

SERVICE="electrosense-sensor-mqtt"
HOST="controller.electrosense.org"
PORT="8883"

# Iteration count
RETRY_FILE="/tmp/wdg_retries"
# Status, if any, to be read from the service upon restart (for logging purposes)
STATUS_FILE="/tmp/wdg_status"

# Only makes sense if service is active
if ! systemctl is-active --quiet $SERVICE
then
  rm -f $RETRY_FILE
  exit 0
fi

# If we find an established connection is fine for us (TCP timeout imposes delay, though)
if netstat -nt | grep $PORT | grep -q ESTABLISHED
then
  rm -f $RETRY_FILE
  exit 0
fi


ITERATIONS=0
[ -e $RETRY_FILE ] && ITERATIONS=$(<$RETRY_FILE)

# Increase and check
ITERATIONS=$(( ITERATIONS + 1 ))
echo "$ITERATIONS" > $RETRY_FILE
if [ ! $ITERATIONS -gt $THRESHOLD ]
then
  # Not yet
  exit 0
fi

# Threshold surpassed

# Check that the remote service is really up (improvement: check TLS protocol)
# timeout -quiet
if ! nc -z -w2 $HOST $PORT || ! openssl s_client -showcerts -connect $HOST:$PORT 2>/dev/null | grep -q "BEGIN CERTIFICATE"
then
  # Service is not reachable or in a valid condition
  # Connectivity problems are not handled on this script
  # BWR: This test (with its counter cleaning) has been moved down in the line so the remote
  # service is not bothered with continuous connection attempts (think local service restart)
  rm -f $RETRY_FILE
  exit 0
fi

# Update status (to be removed from the local service)
echo "Service running at $HOST:$PORT was not being reached from $SERVICE." > $STATUS_FILE

# At this point we have a problem in our local service, let's do the honors:
if ! systemctl restart $SERVICE
then
  # Beyond repair, recommending reboot
  exit -1
fi

# Start over
rm -f $RETRY_FILE
exit 0

